#define MOZ_SOURCE_STAMP 2991f214d4f4d8a3c5cfd95e6223f0660006767d
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/2991f214d4f4d8a3c5cfd95e6223f0660006767d
